package de.juplo.facebook.exceptions;


import com.fasterxml.jackson.core.JsonProcessingException;
import static de.juplo.facebook.exceptions.GraphApiException.OBJECT_MAPPER;
import de.juplo.facebook.exceptions.GraphApiException.Type;
import java.io.IOException;
import org.junit.Test;
import static org.junit.Assert.assertEquals;



/**
 * @author Kai Moritz
 */
public class FacebookErrorMessageMappingTest
{
  final String example =
      "{" +
          "\"error\":{" +
              "\"message\":\"Message describing the error\"," +
              "\"type\":\"OAuthException\"," +
              "\"code\":190," +
              "\"error_subcode\":460," +
              "\"error_user_title\":\"A title\"," +
              "\"error_user_msg\":\"A message\"," +
              "\"fbtrace_id\":\"EJplcsCHuLu\"" +
          "}" +
      "}";


  @Test
  public void testSerialize() throws JsonProcessingException
  {
    FacebookErrorMessage error = new FacebookErrorMessage();
    error.message = "Message describing the error";
    error.type = Type.OAuthException.name();
    error.code = 190;
    error.subCode = 460;
    error.userTitle = "A title";
    error.userMessage = "A message";
    error.traceId = "EJplcsCHuLu";

    assertEquals(example, OBJECT_MAPPER.writeValueAsString(error));
  }

  @Test
  public void testDeserialize() throws IOException
  {
    FacebookErrorMessage error =
        OBJECT_MAPPER.readValue(example, FacebookErrorMessage.class);

    assertEquals("Message describing the error", error.message);
    assertEquals(Type.OAuthException.name(), error.type);
    assertEquals(new Integer(190), error.code);
    assertEquals(new Integer(460), error.subCode);
    assertEquals("A title", error.userTitle);
    assertEquals("A message", error.userMessage);
    assertEquals("EJplcsCHuLu", error.traceId);
  }
}
